/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




package org.greenstone.gatherer.gui;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;
import org.greenstone.gatherer.Gatherer;
/** A JEditorPane subclass that internally supports the undoable interface
 * and allows a designer to specify if the component should recieve focus
 * on a tab event.
 */
public class GEditorPane
    extends JEditorPane 
    implements ActionListener, FocusListener, UndoableEditListener {

    private boolean focusable = true;
	 
    private File file = null;

    private Gatherer gatherer = null;

    private JButton redo_button = null;
    private JButton undo_button = null;

    private JEditorPane editor  = null;

    private UndoManager undo    = null;

    /** This constructor simply sets whether this pane is focusable.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(boolean focusable) {
	this.focusable = focusable;
    }

    /** This constructor sets up the initial JEditorPane settings, and
     * creates the necessary classes to support undo actions.
     * @param gatherer A reference to Gatherer for purposes of messaging
     * and dictionary.
     */
    public GEditorPane(Gatherer gatherer) {
	this.gatherer = gatherer;
	//this.setContentType("text/html");
	this.setEnabled(true);
	this.setFont(new Font("Courier New", Font.PLAIN, 12));
	// Undo support.
	undo = new UndoManager();
	// Create components.
	redo_button = new JButton(gatherer.dictionary.get("UndoManager.Redo"));
	redo_button.addActionListener(this);
	redo_button.setEnabled(false);
	undo_button = new JButton(gatherer.dictionary.get("UndoManager.Undo"));
	undo_button.addActionListener(this);
	undo_button.setEnabled(false);
    }

    /** This initially calls GEditorPane(focusable), but then uses
     * the given file to find the initial content of this pane. Moreover
     * it will now respond to loss of focus events by saving the document
     * content to this file.
     * this pane is focusable.
     * @param file A File to be used as a data source/destination.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(File file, boolean focusable) {
	this(focusable);
	loadFile(file);
	this.addFocusListener(this);
    }

    /** This initially calls GEditorPane(gatherer), but then sets whether
     * this pane is focusable.
     * @param gatherer A reference to Gatherer for purposes of messaging
     * and dictionary.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(Gatherer gatherer, boolean focusable) {
	this(gatherer);
	this.focusable = focusable;
    }

    /** This initially calls GEditorPane(focusable), but then sets 
     * the initial content for this panel.
     * this pane is focusable.
     * @param content A String indicating the initial content of this pane.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(String content, boolean focusable) {
	this(focusable);
	this.setText(content);
    }

    /** This initially calls GEditorPane(gatherer, focusable), but then sets 
     * the initial content for this panel.
     * this pane is focusable.
     * @param gatherer A reference to Gatherer for purposes of messaging
     * and dictionary.
     * @param content A String indicating the initial content of this pane.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(Gatherer gatherer, String content, boolean focusable) {
	this(gatherer, focusable);
	this.setText(content);
    }

    /** This initially calls GEditorPane(gatherer, focusable), but then uses
     * the given file to find the initial content of this pane. Moreover
     * it will now respond to loss of focus events by saving the document
     * content to this file.
     * this pane is focusable.
     * @param gatherer A reference to Gatherer for purposes of messaging
     * and dictionary.
     * @param file A File to be used as a data source/destination.
     * @param focusable A Boolean indicating whether this component should
     * be focusable.
     */
    public GEditorPane(Gatherer gatherer, File file, boolean focusable) {
	this(gatherer, focusable);
	loadFile(file);
	this.addFocusListener(this);
    }

    /** Any implementation of ActionListener must include this method so we
     * can be informed whenever an action has occured.
     * @param event An ActionEvent.
     */
    public void actionPerformed(ActionEvent event) {
	try {
	    if(event.getSource() == redo_button) {
		undo.redo();
	    }
	    else {
		undo.undo();
	    }
	}
	catch (CannotRedoException error) {
	    error.printStackTrace();
	}
	catch (CannotUndoException error) {
	    error.printStackTrace();
	}
	// Update controls to reflect change.
	redo_button.setEnabled(undo.canRedo());
	undo_button.setEnabled(undo.canUndo());
    }		  

    /** Any implementation of FocusListener must include this method so we
     * can be notified when focus is gained.
     * @param event A FocusEvent.
     */
    public void focusGained(FocusEvent event) {
    }

    /** Any implementation of FocusListener must include this method so we
     * can be notified when focus is lost.
     * @param event A FocusEvent.
     */
    public void focusLost(FocusEvent event) {
	if(file != null) {
				// Save the document to file.
	    try {
		FileOutputStream out = new FileOutputStream(file);
		out.write(this.getText().getBytes());
		out.close();
	    }
	    catch (Exception error) {
		error.printStackTrace();
	    }
	}
    }

    /** Retrieves the redo button for this edit pane.
     * @return A JButton.
     */
    public JButton getRedoButton() {
	return redo_button;
    }

    /** Retrieves the undo button for this edit pane.
     * @return A JButton.
     */
    public JButton getUndoButton() {
	return undo_button;
    }

    /** This method is called by any window manager to determine if this
     * particular component is focusable.
     * @return A Boolean specifying if this component is focus traversable.
     * @deprecated
     */
    /** public boolean isFocusTraversable() {
	return focusable;
	}
    */

    /** Loads the contents of the specified file into the document.
     * @param file The target File.
     */
    public void loadFile(File file) {
	// Load the file.
	Gatherer.println("GEditorPane.loadFile():Opening config file " + file.getAbsolutePath());
	try {
	    FileInputStream in = new FileInputStream(file);
	    int character = -1;
	    String content = "";
	    while((character = in.read()) != -1) {
		content = content + (char)character;
	    }
	    in.close();
	    this.file = file;
	    this.setText(content);
	}
	catch (Exception error) {
	    //error.printStackTrace();
	    Gatherer.println("GEditorPane.loadFile():Error: No Such File.");
	}
    }

    /** Any implementation of Undoable must include this method so we can
     * be informed when an undoable event has occured.
     * @param event An UndoableEditEvent.
     */
    public void undoableEditHappened(UndoableEditEvent event) {
	if(undo != null) {
	    undo.addEdit(event.getEdit());
				// Update controls to reflect change.
	    redo_button.setEnabled(undo.canRedo());
	    undo_button.setEnabled(undo.canUndo());
	}
    }
}
