###########################################################################
#
# BasClas.pm -- base class for all classifiers
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 2000 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package BasClas;

# How a classifier works.  
#
# For each classifier requested in the collect.cfg file, buildcol.pl creates
# a new classifier object (a subclass of BasClas).  Later, it passes each 
# document object to each classifier in turn for classification.
#
# Four primary functions are used:
#
# 1. "new" is called before the documents are processed to set up the
#    classifier.
#
# 2. "init" is called after buildcol.pl has created the indexes etc but
#    before the documents are classified in order that the classifier might
#    set any variables it requires, etc.
#
# 3. "classify" is called once for each document object.  The classifier
#    "classifies" each document and updates its local data accordingly.
#
# 4. "get_classify_info" is called after every document has been
#    classified.  It collates the information about the documents and
#    stores a reference to the classifier so that Greenstone can later
#    display it.

# 09/05/02 Added usage datastructure - John Thompson

use parsargv;

my $verbosity_list = 
    [ { 'name' => "0",
	'desc' => "No output." },
      { 'name' => "1",
 	'desc' => "A little bit of output." },
      { 'name' => "2",
 	'desc' => "Some output." },
      { 'name' => "3",
	'desc' => "Lots of output." } ];

my $arguments = 
    [ { 'name' => "builddir",
	'desc' => "Where to put the built indexes.",
	'type' => "string",
	'deft' => "" },
      { 'name' => "outhandle",
	'desc' => "The file handle to write output to.",
	'type' => "string",
	'deft' => "STDERR" },
      { 'name' => "verbosity",
	'desc' => "Controls the quantity of output.",
	'type' => "enum",
	'list' => $verbosity_list, 
	'deft' => "2",
	'reqd' => "no" } ];

my $options = { 'name'     => "BasClas",
		'desc'     => "Base class for all the classifiers.",
		'inherits' => "No",
		'args'     => $arguments };

sub print_xml_usage {
    my $self = shift (@_);
    print STDERR "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n";
    $self->print_xml();
}

sub print_xml {
    my $self = shift (@_);
    my $option_list = $self->{'option_list'};
    my $option = pop( @{$option_list} );
    if(defined $option) 
    {
	print STDERR "<ClassInfo>\n";
	print STDERR "  <Name>$option->{'name'}</Name>\n";
	print STDERR "  <Desc>$option->{'desc'}</Desc>\n";
	print STDERR "  <Inherits>$option->{'inherits'}</Inherits>\n";
	print STDERR "  <Arguments>\n";
	if(defined $option->{'args'})
	{
	    my $args = $option->{'args'};
	    my $x;
	    foreach $x ( @{$args} )
	    {
		print STDERR "    <Option>\n";
		print STDERR "      <Name>$x->{'name'}</Name>\n";
		print STDERR "      <Desc>$x->{'desc'}</Desc>\n";
		print STDERR "      <Type>$x->{'type'}</Type>\n";
		print STDERR "      <Required>$x->{'reqd'}</Required>\n";
		if(defined $x->{'list'}) 
		{
		    print STDERR "      <List>\n";
		    my $list = $x->{'list'};
		    my $y;
		    foreach $y ( @{$list} ) 
		    {
			print STDERR "        <Value>\n";
			print STDERR "          <Name>$y->{'name'}</Name>\n";
			print STDERR "          <Desc>$y->{'desc'}</Desc>\n";
			print STDERR "        </Value>\n";
		    }
		    # Special case of 'input_encoding'
		    if( $x->{'name'} =~ m/^input_encoding$/i ) {
			my $e = $encodings::encodings;
			foreach my $enc (sort {$e->{$a}->{'name'} cmp $e->{$b}->{'name'}} keys (%$e)) {
			    print STDERR "        <Value>\n";
			    print STDERR "          <Name>$enc</Name>\n";
			    print STDERR "          <Desc>$e->{$enc}->{'name'}</Desc>\n";
			    print STDERR "        </Value>\n";
			}
		    }
		    print STDERR "      </List>\n";
		}
		if(defined $x->{'deft'}) 
		{
		    print STDERR "      <Default>$x->{'deft'}</Default>\n";
		}
		print STDERR "    </Option>\n";
	    }
	}
	if(defined $option_list) {
	    $self->print_xml();
	}

	print STDERR "  </Arguments>\n";
	print STDERR "</ClassInfo>\n";
    }
}


sub new_print_usage
{
    local $self = shift(@_);

    # Print the usage message for a classifier (recursively)
    local $descoffset = $self->determine_description_offset(0);
    $self->print_classifier_usage($descoffset, 1);
}


sub determine_description_offset
{
    local $self = shift(@_);
    local $maxoffset = shift(@_);

    local $optionlistref = $self->{'option_list'};
    local @optionlist = @$optionlistref;
    local $classifieroptions = pop(@$optionlistref);
    return $maxoffset if (!defined($classifieroptions));

    # Find the length of the longest option string of this classifier
    local $classifierargs = $classifieroptions->{'args'};
    if (defined($classifierargs)) {
	foreach $option (@$classifierargs) {
	    local $optionname = $option->{'name'};
	    local $optiontype = $option->{'type'};

	    local $optiondescoffset = 3 + length($optionname);
	    if ($optiontype ne "flag") {
		$optiondescoffset = $optiondescoffset + 2 + length($optiontype) + 1;
	    }

	    # Remember the longest
	    if ($optiondescoffset > $maxoffset) {
		$maxoffset = $optiondescoffset;
	    }
	}
    }

    # Recurse up the classifier hierarchy
    $maxoffset = $self->determine_description_offset($maxoffset);
    $self->{'option_list'} = \@optionlist;
    return $maxoffset;
}


sub print_classifier_usage
{
    local $self = shift(@_);
    local $descoffset = shift(@_);
    local $isleafclass = shift(@_);

    local $optionlistref = $self->{'option_list'};
    local @optionlist = @$optionlistref;
    local $classifieroptions = pop(@$optionlistref);
    return if (!defined($classifieroptions));

    local $classifiername = $classifieroptions->{'name'};
    local $classifierargs = $classifieroptions->{'args'};

    # Produce the usage information using the data structure above
    if ($isleafclass) {
	print STDERR " usage: classify $classifiername [options]\n\n";
    }

    # Display the classifier options, if there are some
    if (defined($classifierargs)) {
	# Calculate the column offset of the option descriptions
	local $optiondescoffset = $descoffset + 2;  # 2 spaces between options & descriptions

	if ($isleafclass) {
	    print STDERR " specific options:\n";
	}
	else {
	    print STDERR " general options (from $classifiername):\n";
	}

	# Display the classifier options
	foreach $option (@$classifierargs) {
	    # Display option name
	    local $optionname = $option->{'name'};
	    print STDERR "  -$optionname";
	    local $optionstringlength = length("  -$optionname");

	    # Display option type, if the option is not a flag
	    local $optiontype = $option->{'type'};
	    if ($optiontype ne "flag") {
		print STDERR " <$optiontype>";
		$optionstringlength = $optionstringlength + length(" <$optiontype>");
	    }

	    # Display the option description
	    local $optiondesc = $option->{'desc'};
	    &display_text_in_column($optiondesc, $optiondescoffset, $optionstringlength, 80);

	    # Show the default value for the option, if there is one
	    local $optiondefault = $option->{'deft'};
	    if (defined($optiondefault)) {
		print STDERR " " x $optiondescoffset;
		print STDERR "Default: " . $optiondefault . "\n";
	    }

	    # If the option has a list of possible values, display these
	    local $optionvalueslist = $option->{'list'};
	    if (defined($optionvalueslist)) {
		print STDERR "\n";
		foreach $optionvalue (@$optionvalueslist) {
		    local $optionvaluename = $optionvalue->{'name'};
		    print STDERR " " x $optiondescoffset;
		    print STDERR "$optionvaluename:";

		    local $optionvaluedesc = $optionvalue->{'desc'};
		    &display_text_in_column($optionvaluedesc, ($optiondescoffset + 2),
					    $optiondescoffset + length($optionvaluename), 80);
		}
	    }

	    # Special case for 'input_encoding'
	    if ($optionname =~ m/^input_encoding$/i) {
		my $e = $encodings::encodings;
		foreach $enc (sort {$e->{$a}->{'name'} cmp $e->{$b}->{'name'}} keys (%$e)) {
		    local $encodingname = $enc;
		    print STDERR " " x $optiondescoffset;
		    print STDERR "$encodingname:";

		    local $encodingdesc = $e->{$enc}->{'name'};
		    &display_text_in_column($encodingdesc, ($optiondescoffset + 2),
					    $optiondescoffset + length($encodingname), 80);
		}
	    }

	    # Add a blank line to separate options
	    print STDERR "\n";
	}
    }

    # Recurse up the classifier hierarchy
    $self->print_classifier_usage($descoffset, 0);
    $self->{'option_list'} = \@optionlist;
}


sub display_text_in_column
{
    local ($text, $columnbeg, $firstlineoffset, $columnend) = @_;

    # Spaces are put *before* words, so treat the column beginning as 1 smaller than it is
    $columnbeg = $columnbeg - 1;

    # Add some padding (if needed) for the first line
    local $linelength = $columnbeg;
    if ($firstlineoffset < $columnbeg) {
	print STDERR " " x ($columnbeg - $firstlineoffset);
    }
    else {
	$linelength = $firstlineoffset;
    }

    # Break the text into words, and display one at a time
    local @words = split(/ /, $text);

    foreach $word (@words) {
	# Unescape '<' and '>' characters
	$word =~ s/&lt;/</g;
	$word =~ s/&gt;/>/g;

	# If printing this word would except the column end, start a new line
	if (($linelength + length($word)) >= $columnend) {
	    print STDERR "\n";
	    print STDERR " " x $columnbeg;
	    $linelength = $columnbeg;
	}

	# Write the word
	print STDERR " $word";
	$linelength = $linelength + length(" $word");
    }

    print STDERR "\n";
}


sub print_general_usage {
    my ($plugin_name) = @_;
    print STDERR "
   -verbosity N    Controls the quantity of output.  
                   Defaults to verbosity of buildcol.pl, which is usually 2.

   (Most general classifier options are set internally by buildcol.)

";
}

# print_usage should be overridden for any sub-classes
sub print_usage {
    print STDERR "
This classifier has no classifier-specific options

";
}

sub new {
    my $class = shift (@_);
    my $name = shift (@_);

    my $self = {};

    $self->{'outhandle'} = STDERR;
    
	 $self->{'option_list'} = [ $options ];

    # general options available to all classifiers
    if (!parsargv::parse(\@_,
			 q^builddir/.*/^, \$self->{'builddir'},
			 q^outhandle/.*/STDERR^, \$self->{'outhandle'},
			 q^verbosity/\d/2^, \$self->{'verbosity'},
			 "allow_extra_options")) {
	
	print STDERR "\nThe $name classifier uses an incorrect general option\n";
	print STDERR "(general options are those available to all classifiers).\n";
	print STDERR "Check your collect.cfg configuration file.\n";
    	&print_general_usage($plugin_name);
	die "\n";
    }



    return bless $self, $class;
}

sub init {
    my $self = shift (@_);
}

sub classify {
    my $self = shift (@_);
    my ($doc_obj) = @_;

    my $outhandle = $self->{'outhandle'};
    print $outhandle "BasClas::classify function must be implemented in sub-class\n";
}

sub get_classify_info {
    my $self = shift (@_);

    my $outhandle = $self->{'outhandle'};
    print $outhandle "BasClas::classify function must be implemented in sub-class\n";
}

1;
